searchData={"items":[{"type":"module","title":"Nvir","doc":"This is the main API for Nvir, an environment variable loader and validator.\n\nThe most useful documentation is generally:\n\n* The [README](README.md#basic-usage) for usage instructions.\n* The the `dotenv!/1` function.","ref":"Nvir.html"},{"type":"function","title":"Nvir.cast/2","doc":"","ref":"Nvir.html#cast/2"},{"type":"type","title":"Nvir.config_opt/0","doc":"","ref":"Nvir.html#t:config_opt/0"},{"type":"function","title":"Nvir.dotenv!/1","doc":"Loads specified env files in the system environment. Intended usage is from\n`config/runtime.exs` in your project\n\nVariables defined in the files will not overwrite the system environment if\nthey are already defined. To overwrite the system env, please list your files\nunder an `:overwrite` key.\n\nThis function takes multiple sources and will select the sources to actually\nload based on system properties.\n\nValid sources are:\n\n* A string, this is an actual file that we want to load.\n* A `{tag, value}` tuple where the tag is an atom and the value is a source.\n  Predefined tags are listed below. Additional tags can be defined with\n  `enable_sources/3`.\n* A list of sources. So a keyword list is a valid source, _i.e._ a list of\n  tagged tuples.\n\nFiles are loaded in order of appearance, in two phases:\n\n* First, files that are not wrapped in an `:overwrite` tagged tuple.\n* Then files that are wrapped in such tuples.\n\nFiles that do not exist are safely ignored.","ref":"Nvir.html#dotenv!/1"},{"type":"function","title":"Examples - Nvir.dotenv!/1","doc":"import Config\n    import Nvir\n\n    # Load a single file\n    dotenv!(\".env\")\n\n    # Load multiple files\n    dotenv!([\".env\", \".env.#{config_env()}\"])\n\n    # Load files depending on environment\n    dotenv!(\n      *: \".env\",\n      dev: \".env.dev\",\n      test: \".env.test\"\n    )\n\n    # Load files with and without overwrite\n    dotenv!(\n      dev: \".env\",\n      test: [\".env\", \".env.test\"],\n      overwrite: [test: \".env.test.local\"]\n    )\n\n    # Overwrite the system with all existing files\n    dotenv!(\n      overwrite: [\n        dev: \".env\",\n        test: [\".env\", \".env.test\", \".env.test.local\"]\n      ]\n    )\n\n    # Totally useless but valid :)\n    dotenv!(test: [test: [test: \".env.test\"]])\n    # Same without wrapping the tuples in lists\n    dotenv!({:test, {:test, {:test, \".env.test\"}}})\n\n    # This will not load the file as `:test` and `:dev` will not be\n    # enabled at the same time\n    dotenv!(dev: [test: \".env.test\"])","ref":"Nvir.html#dotenv!/1-examples"},{"type":"function","title":"Predefined tags - Nvir.dotenv!/1","doc":"Tags are enabled under different circumstances.\n\n#### Always enabled\n\n* `:*` -  Acts as a syntactic sugar to avoid mixing keyword lists and strings.\n\n#### Mix environment\n\n* `:dev` - When `Config.config_env()` or `Mix.env()` is `:dev`.\n* `:test` - When `Config.config_env()` or `Mix.env()` is `:test`.\n\nThere is no predefined tag for `:prod` as using .env files in production is\nstrongly discouraged.\n\n#### Continuous integration\n\n* `:ci` - When the `CI` environment variable is `\"true\"`.\n* `:ci@github` - When the `GITHUB_ACTIONS` environment variable is `\"true\"`.\n* `:ci@travis` - When the `TRAVIS` environment variable is `\"true\"`.\n* `:ci@circle` - When the `CIRCLECI` environment variable is `\"true\"`.\n* `:ci@gitlab` - When the `GITLAB_CI` environment variable is `\"true\"`.\n\n#### Operating system\n\n* `:linux` - On Linux machines.\n* `:windows` - On Windows machines.\n* `:darwin` - On MacOS machines.","ref":"Nvir.html#dotenv!/1-predefined-tags"},{"type":"function","title":"Nvir.dotenv!/2","doc":"Same as `dotenv!/1` but accepts a custom configuration to load env files.","ref":"Nvir.html#dotenv!/2"},{"type":"function","title":"Nvir.dotenv_configure/2","doc":"Updates the given configuration with the given options.\n\nThe options are not merged.","ref":"Nvir.html#dotenv_configure/2"},{"type":"function","title":"Example - Nvir.dotenv_configure/2","doc":"Implement loading a custom `:docs` environment and load a file when running a\nrelease:\n\n    import Nvir\n\n    dotenv_loader()\n    |> dotenv_configure(\n      enabled_sources: %{\n        # Enable sources tagged with :docs depending on an environment variable\n        docs: System.get_env(\"MIX_ENV\") == \"docs\",\n\n        # Enable sources tagged with :rel when running a release\n        rel: env!(\"RELEASE_NAME\", :boolean, false)\n      },\n\n      # Load env files relative to this directory\n      cd: \"~/projects/apps/envs\"\n    )\n    |> dotenv!(\n      docs: \".env.docs\",\n      rel: \"releases.env\",\n      dev: \".env.dev\",\n      test: \".env.test\"\n    )","ref":"Nvir.html#dotenv_configure/2-example"},{"type":"function","title":"Nvir.dotenv_loader/0","doc":"Returns the default configuration for the `dotenv!/2` function.","ref":"Nvir.html#dotenv_loader/0"},{"type":"function","title":"Examples - Nvir.dotenv_loader/0","doc":"Implement loading a custom `:docs` environment:\n\n    import Config\n    import Nvir\n\n    dotenv_loader()\n    |> enable_source(:docs, config_env() == \"docs\")\n    |> dotenv!(\n      docs: \".env.docs\",\n      dev: \".env.dev\",\n      test: \".env.test\"\n    )\n\nDisable loading\n\n    import Config\n    import Nvir\n\n    dotenv_loader()\n    |> enable_source(:docs, config_env() == \"docs\")\n    |> dotenv!(\n      docs: \".env.docs\",\n      dev: \".env.dev\",\n      test: \".env.test\"\n    )","ref":"Nvir.html#dotenv_loader/0-examples"},{"type":"function","title":"Nvir.dotenv_new/0","doc":"Returns a configuration for `dotenv!/2` without any enabled source.\n\nGenerally this is used for custom loading strategies, see `dotenv_loader/0` or\n`dotenv_loader/1` to use reasonable defaults.","ref":"Nvir.html#dotenv_new/0"},{"type":"function","title":"Nvir.enable_sources/2","doc":"","ref":"Nvir.html#enable_sources/2"},{"type":"function","title":"Nvir.enable_sources/3","doc":"Enables or disables environment variable sources under the given tag.\n\nFor instance, the following call will load both files:\n\n    Nvir.dotenv_loader()\n    |> Nvir.enable_sources(:custom, true)\n    |> Nvir.dotenv!([\"global.env\", {:custom,  \"local.env\"}])\n\nWhereas the following call will only load files that are not wrapped in a tag.\n\n    Nvir.dotenv_loader()\n    |> Nvir.dotenv!([\"global.env\", {:custom,  \"local.env\"}])\n\nIt is also possible to disable some defaults by overriding them. In the\nfollowing code, the `.env.test` file will never be loaded:\n\n    Nvir.dotenv_loader()\n    |> Nvir.enable_sources(:test, false)\n    |> Nvir.dotenv!([\"global.env\", dev: \".env.dev\", test: \".env.test\"])","ref":"Nvir.html#enable_sources/3"},{"type":"function","title":"Nvir.env!/2","doc":"Returns the value of the given `var`, transformed and validated by the given\n`caster`.\n\nRaises if the variable is not defined or if the caster validation fails.\n\nPlease see the [README](README.md#available-casters) for available casters.","ref":"Nvir.html#env!/2"},{"type":"function","title":"Nvir.env!/3","doc":"Returns the value of the given `var`, transformed and validated by the given\n`caster`.\n\nReturns the `default` value if the variable is not defined.\n\nRaises if the caster validation fails.\n\nPlease see the [README](README.md#available-casters) for available casters.","ref":"Nvir.html#env!/3"},{"type":"function","title":"Nvir.interpolate_var/2","doc":"Takes a parsed value returned by the parser implementation, and a resolver\nfor the interpolated variables.","ref":"Nvir.html#interpolate_var/2"},{"type":"function","title":"Example - Nvir.interpolate_var/2","doc":"iex> envfile = \"\"\"\n    iex> GREETING=Hello $NAME!\n    iex> \"\"\"\n    iex> {:ok, [{\"GREETING\", variable}]} = Nvir.Parser.parse(envfile)\n    iex> resolver = fn \"NAME\" -> \"World\" end\n    iex> Nvir.interpolate_var(variable, resolver)\n    \"Hello World!\"","ref":"Nvir.html#interpolate_var/2-example"},{"type":"type","title":"Nvir.source/0","doc":"","ref":"Nvir.html#t:source/0"},{"type":"type","title":"Nvir.sources/0","doc":"","ref":"Nvir.html#t:sources/0"},{"type":"type","title":"Nvir.t/0","doc":"","ref":"Nvir.html#t:t/0"},{"type":"module","title":"Nvir.Cast","doc":"Collection of casters for environment variables.","ref":"Nvir.Cast.html"},{"type":"function","title":"Nvir.Cast.cast/2","doc":"Casts an environment variable to the desired type.\n\nEnvironment variables are always defined as a string. Thus, the `cast/2`\nfunction will only work with strings as it's first argument.\n\nPlease refer to the \"Available Casters\" of the [README](README.md#available-casters) for a\ndetailed explanation of all casters.","ref":"Nvir.Cast.html#cast/2"},{"type":"type","title":"Nvir.Cast.caster/0","doc":"","ref":"Nvir.Cast.html#t:caster/0"},{"type":"type","title":"Nvir.Cast.result/0","doc":"","ref":"Nvir.Cast.html#t:result/0"},{"type":"behaviour","title":"Nvir.Parser","doc":"A simple .env file parser.","ref":"Nvir.Parser.html"},{"type":"type","title":"Nvir.Parser.buffer/0","doc":"","ref":"Nvir.Parser.html#t:buffer/0"},{"type":"type","title":"Nvir.Parser.key/0","doc":"","ref":"Nvir.Parser.html#t:key/0"},{"type":"function","title":"Nvir.Parser.parse/1","doc":"Returns a list of `{key, value}` for all variables in the given content.\n\nThis function only parses strings, and will not attempt to read from a path.\n\nEach returned value is either a string, or a list of chunks that are either a\nbinary or a `{:var, name}` tuple. Those values can be used with\n`Nvir.interpolate_var/2` by providing a resolver calback that returns the\nvalue of previous variables.","ref":"Nvir.Parser.html#parse/1"},{"type":"function","title":"Resolver example - Nvir.Parser.parse/1","doc":"iex> file_contents = \"GREETING=$INTRO $WHO!\"\n    iex> {:ok, [{\"GREETING\", template}]} = Nvir.Parser.parse(file_contents)\n    iex> resolver = fn\n    ...>   \"INTRO\" -> \"Hello\"\n    ...>   \"WHO\" -> \"World\"\n    ...> end\n    iex> Nvir.interpolate_var(template, resolver)\n    \"Hello World!\"\n\nWhen working with the system env you will likely use `&System.get_env(&1, \"\")`\nas a resolver. It is common to use an empty string for undefined system\nvariables, but you can of course raise from your function if it better suits\nyour needs.","ref":"Nvir.Parser.html#parse/1-resolver-example"},{"type":"callback","title":"Nvir.Parser.parse_file/1","doc":"Must rturn a list of variable definitions in a result tuple.\n\nVariables definitions are defined as lists of `{key, value}` tuples where the\n`value` is either a string or a list of string and `{:var, string}` tuples.\n\nFor instance, given this file content:\n\n```bash\n# .env\nWHO=World\nGREETING=Hello $WHO!\n```\n\nThe `c:parse_file/1` callback should return the following:\n\n```elixir\n{:ok,\n  [\n    {\"WHO\", \"World\"},\n    {\"GREETING\", [\"Hello \", {:var, \"WHO\"}, \"!\"]}\n  ]}\n```\n\nThere is no need to handle different interpolation scenarios at the parser\nlevel. This env file:\n\n```bash\nPATH=b\nPATH=$PATH:c\nPATH=a:$PATH\n```\n\nShould produce the following:\n\n```elixir\n{:ok,\n  [\n    {\"PATH\", \"b\"},\n    {\"PATH\", [{:var, \"PATH\"}, \":c\"]},\n    {\"PATH\", [\"a:\", {:var, \"PATH\"}]}\n  ]}\n```\n\nInterpolation will be handled by `Nvir.dotenv!/1` when variables will be\napplied.","ref":"Nvir.Parser.html#c:parse_file/1"},{"type":"type","title":"Nvir.Parser.parser/0","doc":"","ref":"Nvir.Parser.html#t:parser/0"},{"type":"type","title":"Nvir.Parser.value/0","doc":"","ref":"Nvir.Parser.html#t:value/0"},{"type":"type","title":"Nvir.Parser.variable/0","doc":"","ref":"Nvir.Parser.html#t:variable/0"},{"type":"exception","title":"Nvir.CastError","doc":"Exception defining an invalid cast. For instance with this env file:\n\n\n    NOT_AN_INT=some string\n\nCalling `Nvir.env!/2` with the `:integer!` caster will raise an\n`Nvir.CastError` exception.\n\n    Nvir.env!(\"NOT_AN_INT\", :integer!)","ref":"Nvir.CastError.html"},{"type":"exception","title":"Nvir.LoadError","doc":"Exception raised when `Nvir` failed to parse an env file.","ref":"Nvir.LoadError.html"},{"type":"exception","title":"Nvir.Parser.ParseError","doc":"Exception representing parse errors from the default parser `Nvir.Parser`.","ref":"Nvir.Parser.ParseError.html"},{"type":"extras","title":"Nvir","doc":"# Nvir [![Hex.pm Version](https://img.shields.io/hexpm/v/nvir?color=4e2a8e)](https://hex.pm/packages/nvir) [![GitHub Actions Workflow Status](https://img.shields.io/github/actions/workflow/status/lud/nvir/elixir.yaml?label=CI)](https://github.com/lud/nvir/actions/workflows/elixir.yaml)\n\nAn easy dotenv implementation for Elixir that provides:\n\n* Simple loading of `.env` files with support for inheritance and interpolation.\n* Strong validation and type casting of environment variables.\n* Environment specific configuration.\n\nThis library is heavily inspired from\n[Dotenvy](https://github.com/fireproofsocks/dotenvy) and provides a similar\nexperience.\n\n\n\n- [Installation](#installation)\n- [Basic Usage](#basic-usage)\n- [File loading](#file-loading)\n  - [Defining the sources](#defining-the-sources)\n  - [Overwrite mechanics](#overwrite-mechanics)\n  - [File load order](#file-load-order)\n  - [Custom loaders](#custom-loaders)\n- [The `env!` functions](#the-env-functions)\n  - [Requiring a variable](#requiring-a-variable)\n  - [Default values](#default-values)\n  - [Available Casters](#available-casters)\n  - [Custom Casters](#custom-casters)\n- [Dotenv File Syntax Cheatsheet](#dotenv-file-syntax-cheatsheet)\n  - [Basic Syntax](#basic-syntax)\n  - [Comments](#comments)\n  - [Quoted Values](#quoted-values)\n  - [Multiline Strings](#multiline-strings)\n  - [Variable Interpolation](#variable-interpolation)\n- [Environment Files Inheritance](#environment-files-inheritance)\n  - [Rules for regular files](#rules-for-regular-files)\n  - [Examples](#examples)\n  - [Important Edge Case](#important-edge-case)\n  - [Rules for overwrite files](#rules-for-overwrite-files)","ref":"readme.html"},{"type":"extras","title":"Installation - Nvir","doc":"As usual, pull the library from your `mix.exs` file.\n\n```elixir\ndef deps do\n  [\n    {:nvir, \"~> 0.10\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Basic Usage - Nvir","doc":"You will generally use `Nvir` from your `config/runtime.exs` file.\n\n* Import the module functions, and call `dotenv!/1` to load your files.\n* Use `env!/2` to require a variable and validate it.\n* Use `env!/3` to provide a default value. Default values are not validated.\n\nNote that you do not have to call `dotenv!/1` to use the `env` functions. You can\nuse this library for validation only.\n\n```elixir\n# runtime.exs\n\n# Import the library\nimport Nvir\n\n# Load your env files for local development\ndotenv!([\".env\", \".env.#{config_env()}\"])\n\n# Configure your different services with the env!/2 and env!/3 functions.\nconfig :my_app, MyAppWeb.Endpoint,\n  # expect values to exist\n  secret_key_base: env!(\"SECRET_KEY_BASE\", :string!),\n  url: [host: env!(\"HOST\", :string!), port: 443, scheme: \"https\"],\n  # or provide a sane default\n  http: [ip: {0, 0, 0, 0}, port: env!(\"PORT\", :integer!, 4000)]\n```\n\nThis is most of what you need to know to start using this library. Below is an\nadvanced guide that covers all configuration and usage options.","ref":"readme.html#basic-usage"},{"type":"extras","title":"File loading - Nvir","doc":"","ref":"readme.html#file-loading"},{"type":"extras","title":"Defining the sources - Nvir","doc":"The `dotenv!/1` function accepts paths to files, either absolute or relative to\n`File.cwd!()` (which points to your app root where `mix.exs` is present).\n\nThere are different possible ways to chose the files to load.\n\n\n#### The classic dotenv experience\n\n```elixir\ndotenv!(\".env\")\n```\n\n\n#### A list of sources\n\nNon-existing files are safely ignored.  Your `.env` file will likely\nnot be present in production, and you may have a `.env.test` file but no\n`.env.dev` file.\n\n```elixir\ndotenv!([\".env\", \".env.#{config_env()}\"])\n```\n\n\n#### Tagged sources\n\nIt is possible to wrap the sources in tagged tuples, to limit the loading of\nthe sources to certain conditions:\n\nIn this example, a different file is loaded depending on the current\n`Config.config_env()`.\n\n```elixir\ndotenv!(\n  dev: \".env\",\n  test: \".env.test\"\n)\n```\n\nThis gives you more control on the files that are loaded, and ensures that no\nfile will be loaded in production if the env files are committed to git by\nmistake and/or included in your releases.\n\nRefer to the documentation of `Nvir.dotenv!/1` to know the default enabled tags.\n\n\n#### Mixing it all together\n\nTagged tuples and lists do not have to contain a single file. They can also\ncontain other valid sources, that is, nested lists or tuples.\n\nFor instance, here in `:test` environment we will load two files, plus an\nadditional file if the `:ci` tag is enabled.\n\n```elixir\ndotenv!(\n  dev: \".env\",\n  test: [\".env\", \".env.test\", ci: \".env.ci\"]\n)\n```\n\nIt is also possible to pass the same key multiple times:\n\n```elixir\ndotenv!([\n  test: \".env.test\",\n  ci: \".env.ci\",\n  test: \".env.test.extra\"\n])\n```","ref":"readme.html#defining-the-sources"},{"type":"extras","title":"Overwrite mechanics - Nvir","doc":"The files loaded by this library will not replace variables already defined in\nthe real environment.\n\nThat is, as your `HOME` variable already exists, defining `HOME=/somewhere/else`\nin and `.env` file will have no effect.\n\nA special tag can be given to `dotenv!/1` to overwrite system variables:\n\n```elixir\ndotenv!([\".env\", overwrite: \".env.local\"])\n```\n\nWith the code above, any variable from `.env` that does not already exists will\nbe added to the system env, but _all_ variables from `.env.local` will be set.\n\nJust like environment specific keys, the `:overwrite` key accepts any nested\nsource types. The following forms are equivalent:\n\n```elixir\ndotenv!(\n  dev: [\".env.dev\", overwrite: \".env.local.dev\"],\n  test: [\".env.test\", overwrite: \".env.local.test\"]\n)\n\ndotenv!(\n  dev: \".env.dev\",\n  test: \".env.test\",\n  overwrite: [dev: \".env.local.dev\", test: \".env.local.test\"]\n)\n```\n\nIn `:dev` environment, the two snippets above would both result in loading\n`\".env.dev\"` then `\".env.local.dev\"`.\n\nThe first level of `:overwrite` will determine which group a file belongs to.\nNesting `:overwrite` tags has no effect. In the following snippet, all files\nexcept `1.env` are overwrite files.\n\n```elixir\ndotenv!(\n  dev: \"1.env\",\n  overwrite: [\"2.env\", dev: [\"3.env\", overwrite: \"4.env\"]]\n)\n```\n\nThe `3.env` file is wrapped in an `:overwrite` tag, indirectly.","ref":"readme.html#overwrite-mechanics"},{"type":"extras","title":"File load order - Nvir","doc":"The `dotenv!/1` function follows a couple rules when loading the different\nsources:\n\n* Files are separated in two groups, \"regular\" and \"overwrites\".\n* Within each group, files are always loaded in order of appearance. This is\n  important for files that reuse variables defined in previous files.\n* The \"regular\" group is loaded first. The files from the \"overwrite\" group will\n  see the variables defined by the \"regular\" group.\n\nThe order of execution is the following:\n\n* Load all regular files in order.\n* Patch system environment with non-existing keys.\n* Load all overwrite files in order.\n* Overwrite system environment with all their values.\n\nThis means that the following expression will **not** load and apply\n`.env.local` first because it belongs to the \"overwrite\" group, which is applied\nlast. But `.env1` will always be loaded before `.env2`.\n\n```elixir\ndotenv!(overwrite: \".env.local\", dev: \".env1\", dev: \".env2\")\n```","ref":"readme.html#file-load-order"},{"type":"extras","title":"Custom loaders - Nvir","doc":"It is possible to customize the way the files are loaded. The order of the files\nis deterministic and cannot be changed, but options exist to change how they are\nloaded.\n\n\n#### Using a custom loader\n\nThe simple way is to start from the default loader and change its properties:\n\n```elixir\n# runtime.exs\nimport Config\nimport Nvir\n\ndotenv_loader()\n|> enable_sources(:docs, config_env() == \"docs\")\n|> enable_sources(:release, env!(\"RELEASE_NAME\", :boolean, false))\n|> dotenv_configure(cd: \"/app/release/env\")\n|> dotenv!(\n  dev: \".env\",\n  test: \".env.test\",\n  docs: \".env.docs\",\n  release: \"/var/release.env\"\n)\n```\n\nIn the example above, we will enable the `:docs` and `:ci` tag when the defined\nconditions are met.\n\nPlus, we changed the directory where the .env files are loaded from. This will\nnot affect the `/var/release.env` file since it's an absolute path.\n\nPlease refer to the documentation of `dotenv_configure/2` to learn more about\nthe available options.\n\n#### Disabling default tags\n\nIt is also possible to redefine predefined tags. Here we replace the `:test` tag\nwith a possibly different boolean value.\n\n```elixir\n# runtime.exs\nimport Config\nimport Nvir\n\ndotenv_loader()\n|> enable_sources(:test, config_env() == :test and MyApp.some_custom_check())\n|> dotenv!(\n  dev: \".env\",\n  test: \".env.test\"\n)\n```\n\nThe `:overwrite` tag cannot be changed, as it is handled separately from other\ntags.\n\n\n#### Disable all tags by default\n\nUse `dotenv_new()` instead of `dotenv_loader()` to get an empty loader without\nany enabled tag.\n\n\n#### Use a custom parser\n\nIf you want to parse the .env files yourself, or add support for other file\nformats, pass an implementation of the `Nvir.Parser` behaviour as the `:parser`\noption:\n\n```elixir\n# runtime.exs\nimport Config\nimport Nvir\n\ndotenv_new()\n|> dotenv_configure(parser: MyApp.YamlEnvParser)\n|> dotenv!(\"priv/dev-env.yaml\")\n```","ref":"readme.html#custom-loaders"},{"type":"extras","title":"The `env!` functions - Nvir","doc":"The `env!` functions allows you to load an environment variable and cast its\ncontent to the appropriate type.","ref":"readme.html#the-env-functions"},{"type":"extras","title":"Requiring a variable - Nvir","doc":"Calling `env!(var, caster)` will attempt to fetch the variable, just like\n`System.fetch_env!/1` does, cast its value, and return it.\n\nA `System.EnvError` exception will be raised if the variable is not defined.\n\nAn `Nvir.CastError` exception will be raised if the cast fails.","ref":"readme.html#requiring-a-variable"},{"type":"extras","title":"Default values - Nvir","doc":"Calling `env!(var, caster, default)` will use the default value if the key is\nnot defined.\n\nThe function will **not** use the default value if the cast of an existing key\nfails. This will still raise an `Nvir.CastError`.\n\nThe default value is not validated, so you can for instance call\n`env!(\"SOME_VAR\", :integer, :infinity)`, whereas `:infinity` is not a valid\ninteger.","ref":"readme.html#default-values"},{"type":"extras","title":"Available Casters - Nvir","doc":"Casters come into three flavors:\n\n* The \"value as is\" one.\n* The \"nil\" one with a `?` suffix that converts empty strings to `nil`. It will\n  however not fallback to the default value given to `env!/3` if the key exists.\n* The \"bang\" one with a `!` suffix that will raise an `Nvir.CastError` exception for\n  empty strings and special cases described below.\n\nIn some languages, using `null` where an integer is expected will cast the value\nto a \"default value\", generally `0` for integers. This is not the case in\nElixir. To respect that, casters for such types behave the same with and without\nthe `!` suffix. Namely, `:integer` and `:float` will raise for empty strings.\n\nIt is however not the case for `:existing_atom`, because the `:\"\"` atom is\ngenerally defined by the system long before an application starts, in Erlang\njust as in Elixir.\n\nEmpty strings occur when a variable is defined without a value:\n\n```\nHOST=localhost # value is \"localhost\"\nPORT=          # value is \"\"\n```\n\n#### String Casters\n\n| Caster | Description |\n|--------|-------------|\n| `:string` | Returns the value as-is. |\n| `:string?` | Converts empty strings to `nil`. |\n| `:string!` | Raises for empty strings. |\n\n#### Boolean Casters\n\n| Caster | Description |\n|--------|-------------|\n| `:boolean` | `\"false\"`, `\"0\"` and empty strings become `false`, any other value is `true`. Case insensitive. It is recommended to use `:boolean!` instead. |\n| `:boolean!` | Accepts only `\"true\"`, `\"false\"`, `\"1\"`, and `\"0\"`. Case insensitive. |\n\n#### Number Casters\n\n| Caster | Description |\n|--------|-------------|\n| `:integer!` | Strict integer parsing. |\n| `:integer?` | Like `:integer!` but empty strings becomes `nil`. |\n| `:float!` | Strict float parsing. |\n| `:float?` | Like `:float!` but empty strings becomes `nil`. |\n\n#### Atom Casters\n\n| Caster | Description |\n|--------|-------------|\n| `:atom` | Converts the value to an atom. Use the `:existing_atom` variants when possible. |\n| `:atom?` | Like `:atom` but empty strings becomes `nil`. |\n| `:atom!` | Like `:atom` but rejects empty strings. |\n| `:existing_atom` | Converts to existing atom only, raises otherwise. |\n| `:existing_atom?` | Like `:existing_atom` but empty strings becomes `nil`. |\n| `:existing_atom!` | Like `:existing_atom` but rejects empty strings. |\n\n#### Deprecated casters\n\nThose exist for legacy reasons and should not be used.\n\n| Caster | Description |\n|--------|-------------|\n| `:boolean?` | Same as `:boolean`. |\n| `:integer` | Same as `:integer!`. |\n| `:float` | Same as `:float!`. |","ref":"readme.html#available-casters"},{"type":"extras","title":"Custom Casters - Nvir","doc":"The second argument to `env!/2` and `env/3` also accept custom validators using an `fn`. The given function must return `{:ok, value}` or `{:error, message}` where `message` is a string.\n\n```elixir\nenv!(\"URL\", fn\n  \"https://\" <> _ = url -> {:ok, url}\n  _ -> {:error, \"https:// is required\"}\nend)\n```\n\nIt is also possible to return directly an error from `Nvir.cast/2`:\n\n```elixir\nenv!(\"PORT\", fn value ->\n  case Nvir.cast(value, :integer!) do\n    {:ok, port} when port > 1024 -> {:ok, port}\n    {:ok, port} -> {:error, \"invalid port: #{port}\"}\n    {:error, reason} -> {:error, reason}\n  end\nend)\n```","ref":"readme.html#custom-casters"},{"type":"extras","title":"Dotenv File Syntax Cheatsheet - Nvir","doc":"","ref":"readme.html#dotenv-file-syntax-cheatsheet"},{"type":"extras","title":"Basic Syntax - Nvir","doc":"```bash\n# Simple assignment\nKEY=value\n\n# With spaces around =\nKEY = value\n\n# Empty value\nEMPTY=\nEMPTY=\"\"\nEMPTY=''\n\n# The parser will ignore an \"export\" prefix\nexport KEY=value\n\n# Interpolation with previously defined variable\nPATH=/usr/bin\nPATH=$PATH:/home/alice/bin\nPATH=/usr/local/bin:$PATH\n```","ref":"readme.html#basic-syntax"},{"type":"extras","title":"Comments - Nvir","doc":"Comments are supported on their own line or at the end of a line.\n\n**Important**, when a value is not quoted, the comment `#` character must be\nseparated by at least one space, otherwise the comment will be included in the\nvalue.\n\n```bash\n# This is a comment on it's own line\n\nKEY=value # Inline comment\n\nKEY=value# No preceding space, this is part of the value\n```","ref":"readme.html#comments"},{"type":"extras","title":"Quoted Values - Nvir","doc":"* Quotes are optional.\n* Double quotes let you write escape sequences.\n* Single quotes define verbatim values. No escaping is done except for the\n  single quote itself.\n\n#### Raw Strings\n```bash\nKEY=raw value with spaces\n```\n\n#### Double Quotes\n```bash\nKEY=\"value with spaces\"\nKEY=\"escape \\\"quotes\\\" inside\"\nKEY=\"supports \\n \\r \\t \\b \\f escapes\"\n```\n\n#### Single Quotes\n```bash\nKEY='value with spaces'\nKEY='no escapes \\n' # value will have a \"\\\" character followed by a \"n\".\nKEY='escape \\'quotes\\' inside'\n```","ref":"readme.html#quoted-values"},{"type":"extras","title":"Multiline Strings - Nvir","doc":"The same rules applies for escaping as in single line values:\n\n* Double quotes let you write escape sequences.\n* Single quotes define verbatim values. No escaping is done except for the\n  single quote itself.\n\n\n#### Triple Double Quotes\n\n```bash\nKEY=\"\"\"\nLine 1\nLine 2 with \"quotes\"\n\"\"\"\n```\n\n#### Triple Single Quotes\n\n```bash\nKEY='''\nLine 1\nLine 2 with 'quotes'\n'''\n```","ref":"readme.html#multiline-strings"},{"type":"extras","title":"Variable Interpolation - Nvir","doc":"`Nvir` supports variable interpolation within env files. Please refer to the\n\"Environment Files Inheritance\" section for more details on which value is used\non different setups.\n\n```bash\n# This variable can be used below in the same file\nGREETING=Hello\n\n# Basic syntax\nMSG=$GREETING World\n\n# Enclosed syntax\nMSG=${GREETING} World\n\n# Not interpolated (single quotes)\nMSG='$GREETING World'\n\n# In raw values, a comment without a preceding space will be included in the\n# value\nMSG=$GREETING# This is part of the value\nMSG=${GREETING}# This too\nMSG=$GREETING # Actual comment\n```","ref":"readme.html#variable-interpolation"},{"type":"extras","title":"Environment Files Inheritance - Nvir","doc":"","ref":"readme.html#environment-files-inheritance"},{"type":"extras","title":"Rules for regular files - Nvir","doc":"These rules apply to the regular group. Overwrite files will always have their\nvalues added to system environment.\n\n1. System environment variables always take precedence over env files.\n2. Multiple env files can be loaded in sequence, with later files overwriting\n   earlier ones\n3. Variable interpolation (`$VAR`) uses values from the system first, then from\n   the most recently defined value.","ref":"readme.html#rules-for-regular-files"},{"type":"extras","title":"Examples - Nvir","doc":"```elixir\n# System state:\n# WHO=world\n\n# .env\nWHO=moon\nHELLO=hello $WHO   # Will use WHO=world from system since we are not overwriting\n\n# Result:\n# HELLO=hello world\n```\n\nWith multiple files, we use the latest value. In this exemple the variable is\nnot already defined in the system:\n\n```elixir\n# .env\nWHO=world\n\n# .env.dev\nWHO=mars\nHELLO=hello $WHO # This defines HELLO=hello mars\n\n# .env.dev.2\nWHO=moon\nHELLO=hello $WHO # this defines HELLO=hello moon\n\n# With loading order of .env, .env.dev, .env.dev.2\n# we will have the following:\n# WHO=moon\n# HELLO=hello moon\n```\n\nSo, \"regular\" files do not overwrite the system environment, but they act as a\ngroup and overwrite themselves as if they were a single file.","ref":"readme.html#examples"},{"type":"extras","title":"Important Edge Case - Nvir","doc":"When a variable using interpolation is not redefined in subsequent files, it\nkeeps using the value from when it was defined.\n\n```elixir\n# .env\nWHO=world\n\n# .env.dev\nWHO=mars\nHELLO=hello $WHO    # HELLO is defined here, uses WHO=mars\n\n# .env.dev.2\nWHO=moon           # WHO is updated, but HELLO keeps its value\n                   # since it's not redefined here\n\n# Final result:\n# WHO=moon\n# HELLO=hello mars  # Not \"hello moon\"!\n```\n\nThis may cause inconsistencies if you code depends on the values of both `HELLO`\nand `WHO`.","ref":"readme.html#important-edge-case"},{"type":"extras","title":"Rules for overwrite files - Nvir","doc":"Overwrite files follow the same logic, each file overwrites the previous ones.\n\nThe only difference is that the values in the files take precedence over any\npreexisting variable in the system environment.\n\nThe edge case documented above still applies.","ref":"readme.html#rules-for-overwrite-files"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}